# Generated by Django 2.0.7 on 2018-07-31 16:34

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='JobLogModel',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(db_index=True, default='', editable=False, max_length=128, verbose_name='name')),
                ('count', models.BigIntegerField(default=0, editable=False, verbose_name='count')),
                ('date_started', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='started')),
                ('date_ended', models.DateTimeField(blank=True, db_index=True, default=None, editable=False, null=True, verbose_name='ended')),
                ('duration', models.DurationField(blank=True, db_index=True, default=None, null=True, verbose_name='duration')),
                ('state', models.CharField(choices=[('running', '▶ running'), ('finished', '✔ finished'), ('error', '❌ error')], db_index=True, default='running', editable=False, max_length=64, verbose_name='state')),
                ('log_text', models.TextField(blank=True, default=None, editable=False, null=True, verbose_name='log')),
                ('error_text', models.TextField(blank=True, default=None, editable=False, null=True, verbose_name='error log')),
            ],
            options={
                'verbose_name_plural': 'Job logs',
                'verbose_name': 'Job log',
            },
        ),
    ]
